#!/usr/bin/perl

use strict;
use Data::Dumper;

#
#	Data files to read
#
my @filelist = qw(
	cadetfila_tc.txt
	cmwa_tc.txt
	cvwa_tc.txt
	scvwa_tc.txt
	junior_tc.txt
);

my $file;			# current file being read
my $output_filename;

my $location;		# where tournament took place
my $age_group;		# age group being processed
my $style;			# folkstyle / freestyle / greco-roman
my $hdr_name;		# column header (location + style)

my $item;			# ubiquitous foreach loop variable
my $name;			# person's full name
my @firstlast;		# array holding split of $name
my $firstname;		# person's first name
my $lastname;		# last name
my $team;			# and team
my $total;			# total number of wins for a person
my @win;			# convenience array - holds "win/no win" list
					# for one competitor
my $i;				# ubiquitous for loop counter

#
#	This is a hash of hash of array
#	key: age group
#	value: reference to an anonymous hash, with:
#
#		key: a string in the form "lastname:firstname:team"
#		value: an array of 0 or 1 for each event (didn't win/won)
my %info = ();

#
#	This is a hash of arrays.
#	key = age group,
#	value = array of events that have results for that age group
#
my %event = ();

my $n_events;		# current number of events for an age group
my $data;			# buffer for data read from files

foreach $file (@filelist)
{
	open INFILE, "<$file" or die("Cannot open $file\n");

	chomp($location = <INFILE>);	# first line contains tournament location
	while ($data = <INFILE>)
	{
		chomp $data;
		if ($data =~ m/^\*/)	# a new age group
		{
			# add the location and style to the list of events for this
			# age group.
			($age_group, $style)= $data =~ m/^\*(.*)\s*\/\s*(.*)$/;
			$style = trim($style);
			$hdr_name = "$location $style";
			push @{$event{$age_group}}, $hdr_name;
			$n_events = scalar @{$event{$age_group}};	# for ease of access
		}
		elsif ($data =~ m/^\s*1\.\s*/)	# we have a first-place winner
		{
			# get rid of the number
			$data =~ s/^\s*1\.\s*//;
			
			# and split off the name and team
			($name, $team) = split(/,/, $data);
			$name = trim($name);
			$team = trim($team);
			
			# split name into first and last name;
			# presume that everything after the first blank is
			# a "last name".
			@firstlast = split(/\s+/, $name);
			$firstname = $firstlast[0];
			shift @firstlast;
			$lastname = join(" ", @firstlast);

			#
			#	This statement does all the work, with autovivification
			#	creating the sub-hash and the array and the
			#	intermediate array elements if required.
			#	
			$info{$age_group}->						# for this age group 
				{"$lastname:$firstname:$team"}->	# and this competitor
					[$n_events - 1]					# and this event number
					= 1;							# set "win" to true
		}
	}
	close INFILE;
}

#
#	All the input files are now processed.
#	Begin output phase.
#
foreach $age_group (sort keys %info)
{
	print STDERR "Output for $age_group...\n";	

	$output_filename = lc($age_group) . ".csv";
	$output_filename =~ s/ /_/g;
	open OUTFILE, ">$output_filename" or
		die("Cannot open $output_filename");

	# print the first row of the output CSV file:
	# lastname, firstname, team headings

	print OUTFILE "Last Name\tFirst Name\tTeam\t";

	# events for ths age group
	print OUTFILE join("\t", @{$event{$age_group}});

	# total column
	print OUTFILE "\tTotal\n";

	# store number of events for convenience
	$n_events = scalar @{$event{$age_group}};
	
	# Sort the competitors by last name
	foreach $item (sort keys %{$info{$age_group}})
	{
		($lastname, $firstname, $team) = split(/:/, $item);
		
		print OUTFILE "$lastname\t$firstname\t$team\t";
		
		# Copy the wins/losses array
		@win = @{$info{$age_group}->{$item}};
		$total = 0;
		for ($i=0; $i < $n_events; $i++)
		{
			if ($win[$i] == 1)
			{
				print OUTFILE "X\t";
				$total++;
			}
			else
			{
				print OUTFILE "\t";
			}
		}
		
		#	output the count
		print OUTFILE "$total\n";
	}
}

close OUTFILE;

sub trim
{
	my $str = shift;
	$str =~ s/^\s+//;
	$str =~ s/\s+$//;
	return $str;
}

